Module fixed

Module fixed 

Buffers pre-registered with the kernel.

This module provides facilities for registering in-memory buffers with the tokio-uring runtime. Operations like File::read_fixed_at and File::write_fixed_at make use of buffers pre-mapped by the kernel to reduce per-I/O overhead.

Two kinds of buffer collections are provided: FixedBufRegistry and FixedBufPool, realizing two different patterns of buffer management. The register method on either of these types is used to register a collection of buffers with the kernel. It must be called before any of the FixedBuf handles to the collection’s buffers can be used with I/O operations.

Re-exports

pub use pool::FixedBufPool;

Modules

pool
A dynamic collection of I/O buffers pre-registered with the kernel.

Structs

FixedBuf
A unique handle to a memory buffer that can be pre-registered with the kernel for io-uring operations.
FixedBufRegistry
An indexed collection of I/O buffers pre-registered with the kernel.